package FDN::Web::FCGI;

use strict;
use Exporter;
use FDN::Common::Config;
use FDN::Common::Logger;
use FDN::Common::Sys;
use base 'FCGI::ProcManager';

my $profile = 'default';

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new(@_);
	$self->__read_config('/etc/fdn/web/fcgi.cfg',$profile);
	return $self;
}

sub profile {
	my $self = shift;
	my $pr = shift;
	$profile = $pr if defined $pr;
	return $profile;
}

sub __read_config {
	my $self = shift;
	my ( $file, $section ) = @_;
	$file = '/etc/fdn/web/fcgi.cfg' if ! defined $file;
	$section = 'default' if ! defined $section;

	my $cfg = Config_Section($file, 'default');
	$cfg = {} if ! defined $cfg;
	if ( defined $section ) {
		my $tmp = Config_Section($file, $section);
		$cfg = { %$cfg, %$tmp } if defined $tmp;
	}

	my $max = $cfg->{'mem_growth'};
	$max //= '50M';
	if ( $max =~ /^(\d+)\s*G\s*$/i ) {
		$max = $1 * 1024*1024*1024;
	} elsif ( $max =~ /^(\d+)\s*M\s*$/i ) {
		$max = $1 * 1024*1024;
	} elsif ( $max =~ /^(\d+)\s*k\s*$/i ) {
		$max = $1 * 1024;
	} elsif ( $max !~ /^(\d+)$/ ) {
		$max = 0;
	}
	$self->{'mem_growth'} = $max;
	
	$max = $cfg->{'max_requests'};
	if ( ! defined $max || $max !~ /^\d+$/ ) {
		$max = 0;
	}
	$self->{'max_requests'} = $max;

	$max = $cfg->{'n_processes'};
	if ( ! defined $max || $max !~ /\d+$/ ) {
		$max = 1;
	}
	$self->{'n_processes'} = $max;
}

# We over-ride this method to let the process name intact, while
# FCGI::ProcManager wants to override it.
sub pm_change_process_name {
}

sub max_requests { shift->pm_parameter('max_requests', @_); }
sub mem_growth { shift->pm_parameter('mem_growth', @_); }

sub handling_init {
	my $self = shift;
	# N'a lieu que pour les process qui sortent de pm_manage, donc
	# pour l'ensemble des fils, avant d'entrer dans la boucle qui
	# accepte les requetes.
	$self->SUPER::handling_init();
	$self->{_request_counter} = $self->max_requests;
	$self->{_mem_growth} = $self->mem_growth;
	$self->{_mem_init} = proc_self_stat('vsize');
}

# Gestionnaire de signaux du process principal, sur un HUP il va terminer
# proprement tous ses fils puis en relancer des tout frais. On en profite
# pour relire la configuration.
sub sig_manager {
	my $self = shift;
	my ( $name ) = @_;
	if ( $name eq 'HUP' ) {
		$self->__read_config('/etc/fdn/web/fcgi.cfg',$profile);
	}
	$self->SUPER::sig_manager($name);
}


sub pm_post_dispatch {
	my $self = shift;
	if ($self->max_requests > 0 && --$self->{_request_counter} == 0) {
		$self->pm_exit("safe exit after max_requests");
	}
	if ( $self->mem_growth > 0 ) {
		if ( proc_self_stat('vsize') > $self->{_mem_init} + $self->mem_growth ) {
			$self->pm_exit("safe exit on mem_growth, became too big");
		}
	}
	$self->SUPER::pm_post_dispatch();
}

sub pm_notify {
	my $self = shift;
	my ( $msg ) = @_;
	$msg =~ s/\s*$//;
	info $msg;
}

1;
