# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Theo Varier
# - Benjamin Bayart
package FDN::Web::TemplateParse;
use strict;
use Exporter;

our @ISA = qw (Exporter);
our @EXPORT    = qw (
	remplace
);
our @EXPORT_OK = qw (
	parse
	ttoh
	$TP_ERREUR
);
use FDN::Common::Logger;

our $TP_ERREUR={};


# pr dfinition justifie par l'appel rcursif de la fonction parse
sub parse ( $$;$ ) ;

######################################################################
# Parseur :
# remplace dans $texte les tags par les valeurs de la table $tags
# $texte contient le texte  parser
# $tags est une rfrence sur la table de hash des balises  remplacer
# $com_tags est une ref sur la table des tags commun  tous les loop
######################################################################
sub parse ( $$;$ ) {
  my ( $texte, $tags, $com_tags ) = @_;

  if( ref($tags) eq 'HASH' ){
    my $local_tags;
    if( defined($com_tags) and (ref($com_tags) eq 'HASH')){
      $local_tags = { %{$tags}, %{$com_tags} };
    } else {
      $local_tags = $tags;
    }

    $texte =~ s{
                ( \\ % ) |
                (
                (\<\!--\ *)? %% ( ( LOOP | IF | NOT ) (-) )? (.*?) ( \6 )? %% (\ *--\>)?
                ( (.*?)
                (\<\!--\ *)? %% \6 \7 \6 E \5 %% (\ *--\>)? )?
               )
              }{
                my ( $op, $borne, $data, $temp, $elem, $pour_cent ) = ( $5, $7, $11  , '', undef, $1 );
                $op = '' if not defined $op;
                if ( exists($local_tags->{$borne}) and defined($local_tags->{$borne}) ) {
                  if ( $op eq 'IF' and $local_tags->{$borne} ) {
                    $temp = parse($data, $tags, $com_tags );
                  }
                  elsif ( $op eq 'NOT' and ! $local_tags->{$borne} ) {
                    $temp = parse($data, $tags, $com_tags );
                  }
                  elsif ( $op eq 'LOOP' and ( ref($local_tags->{$borne}) eq 'ARRAY' ) ) {
                    foreach $elem (@{$local_tags->{$borne}}) {
                      $temp .= parse($data, $elem, $com_tags );
                    }
                  }
                  elsif ( ! $op ) {
                    $temp = $local_tags->{$borne};
                  }
                }
                elsif ( $op eq 'NOT' ) {
                  $temp = parse($data, $tags, $com_tags );
                }
                elsif ( defined($pour_cent) ) {
                  $temp = '%';
                }
                "$temp";
              }gexs;
    return $texte;
  } else {
    $TP_ERREUR->{'code'} = 2;
    $TP_ERREUR->{'desc'} = "erreur de type de tags\n";
    $TP_ERREUR->{'tags'} = $tags;
    return undef ;
  }
}


####################################################################
# remplace :
# met le contenu du fichier template dans une variable et appelle le
# parseur
# retourne le contenu  afficher
# $template est le handle du template
# $tags est une ref sur la table de hash des balises  remplacer
# Ajout d'un 3 me paramtre pour les tags communs !
####################################################################
sub remplace ( $$;$ ) {
  my ( $template, $tags, $com_tags ) = @_;
  my $text;
  local *TEMPL;
  $TP_ERREUR = {};

  if( open( TEMPL, "$template" ) ) {
    local $/;    #pour avaler le fichier (la modification de la variable reste locale)
    $text = <TEMPL>;
    close(TEMPL);    
    return  &parse( $text, $tags, $com_tags ) ;
  } else {
    $TP_ERREUR->{'code'} = 1;
    $TP_ERREUR->{'desc'} = "erreur d'ouverture du fichier template\n";
    $TP_ERREUR->{'template'} = $template;
    error "Erreur d'ouverture du fichier template $template";
    return undef ;
  }
}


####################################################
# fonction de construction d'une table de hash adoc  partir d'un tableau
#  afficher
# Prend en argument la reference du tableau, le nom du champ colonne
# le nom de l'lment, le nombre de colonne(s), un flag qui dit de complter 
# la table avec des champs vide si la dernire ligne est incomplte, 
# une table de hash d'lments communs
# Retourne la rfrence sur le taleau,  mettre dans la table de hash.
####################################################

sub ttoh {
  my (
    $tableau,   $colname, $elemname, $ncol,
    $completer, $commun,  $funct,    $nomfunct
    )
    = @_;
  my @atable;
  my ( $elem, $col, $li );

  if ($completer) {
    my $i;
    for ( $i = 0 ; $i < ( $#{$tableau} % $ncol ) ; $i++ ) {
      push @{$tableau}, "";
    }
  }

  unless ( defined($funct) and ref($funct) eq 'CODE' ){
    $funct = sub { 1; }
  }

  $col = $li = 0;

  foreach $elem ( @{$tableau} ) {

    if ( ref($elem) eq 'HASH' ){
      $atable[$li]->{$colname}[$col] = {
                                        $nomfunct => &{$funct} ($elem),
                                        col       => $col,
                                        li        => $li,
                                        %{$elem}
                                       };
    } else {
      $atable[$li]->{$colname}[$col] = {
                                        $elemname => $elem,
                                        $nomfunct => &{$funct} ($elem),
                                        col       => $col,
                                        li        => $li
                                       };
    }


    if ( $li % 2 == 0 ) {
      $atable[$li]->{$colname}[$col]{lipaire}[0] =
        $atable[$li]->{$colname}[$col];
    }
    else {
      $atable[$li]->{$colname}[$col]{liimpaire}[0] =
        $atable[$li]->{$colname}[$col];
    }

    if ( $col % 2 == 0 ) {
      $atable[$li]->{$colname}[$col]{colpaire}[0] =
        $atable[$li]->{$colname}[$col];
    }
    else {
      $atable[$li]->{$colname}[$col]{colimpaire}[0] =
        $atable[$li]->{$colname}[$col];
    }

    if( ref($commun) eq 'HASH' ){
      $atable[$li]->{$colname}[$col] = { %{$atable[$li]->{$colname}[$col]}, %{$commun} };
    }

    if ( $col == ( $ncol - 1 ) ) {
      $col = 0;
      $li += 1;
    }
    else {
      $col += 1;
    }

  }
  return \@atable;
}

1;

__END__

la doc a t dplace dans TemplateParse.pod

