package FDN::Web::Utils;

use strict;
use Exporter;

our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	eject_page_html
	eject_page_text
	
	send_html_file
	send_png_file
	send_jpeg_file
	send_pdf_file
	send_ps_file
);
our @EXPORT_OK = qw (
	__send_file
);

use FDN::Common::Logger;

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

sub eject_page_html($) {
	my ( $page ) = @_;
	print "Content-Type: text/html; charset=ISO-8859-1\n\n";
	print $page;
}

sub eject_page_text($) {
	my ( $page ) = @_;
	print "Content-Type: text/plain\n\n";
	print $page;
}

sub __asname($;$) {
	my ( $file, $asname ) = @_;
	if ( ! defined $asname ) {
		$asname = $file;
		$asname =~ s/^(.*)?\/([^\/]+)$/$2/;
	}
	return $asname;
}

sub __send_file($$;$) {
	my ( $file, $mime, $asname ) = @_;
	$asname = __asname($file, $asname);
	print "Content-Type: $mime\n";
	print "Content-Disposition: inline; filename=\"$asname\"\n\n";
	__debug "Sending $file as $mime with name $asname";
	local $/;
	open FILE, $file;
	print <FILE>;
	close FILE;
}

sub send_html_file($;$) {
	my ( $file, $asname ) = @_;
	__send_file($file,'text/html; charset=ISO-8859-1',$asname);
}

sub send_png_file($;$) {
	my ( $file, $asname ) = @_;
	__send_file($file,'image/png',$asname);
}

sub send_jpeg_file($;$) {
	my ( $file, $asname ) = @_;
	__send_file($file,'image/jpeg',$asname);
}

sub send_pdf_file($;$) {
	my ( $file, $asname ) = @_;
	__send_file($file,'application/pdf',$asname);
}

sub send_ps_file($;$) {
	my ( $file, $asname ) = @_;
	__send_file($file,'application/postscript', $asname);
}

1;

